


from pandasflow.split.train_test import train_test
from pandasflow.split.train_valid_test import train_valid_test

from pandasflow.split.inherit_train_test import inherit_train_test
from pandasflow.split.inherit_train_valid_test import inherit_train_valid_test


import pandas as pd
import pandasflow as pdf
from sklearn.metrics import confusion_matrix, roc_auc_score, average_precision_score



def metrics_class(y_true, y_pred, round_=2):
	tn, fp, fn, tp = confusion_matrix(y_true, y_pred).ravel()
	roc_auc = round(roc_auc_score(y_true, y_pred), round_)
	precrec = round(average_precision_score(y_true, y_pred), round_)
	
	table = pd.DataFrame()
	table.index = ['ROCAUC', 'PRECREC']
	table[' '] = [roc_auc, precrec]
	
	print(f'CONFMAT  {(tn, fp, fn, tp)}', table)





from pandasflow.metrics.metrics_class import metrics_class

from pandasflow.metrics.mean_error import mean_error

from pandasflow.metrics.lloss_up import lloss_up

from pandasflow.metrics.conf_mat import conf_mat

from pandasflow.metrics.precision_recall import precision_recall

from pandasflow.metrics.roc_auc import roc_auc


import pandasflow as pdf
import seaborn as sns


def err_mean_diff(df, index_col, dark=True, color='red'):
	CM = sns.dark_palette(color, as_cmap=True) if dark else sns.light_palette(color, as_cmap=True)
	
	t = df.groupby(index_col)[['metrics', 'error_abs']].agg(['count', 'mean'])
	t = pdf.reset_mi(t)
	t = t.drop(['error_abs_count'], axis=1)
	t = t.rename({'error_count': 'count'}, axis=1)
	t['error_mean_diff'] = t['error_abs_mean'] - df['error_abs'].mean()
	t = t.rename({'error_count': 'count'}, axis=1)
	t['test_mean_error'] = df['error_abs'].mean()
	
	return t.style.background_gradient(cmap=CM, subset=['error_mean_diff'])














import pandasflow as pdf
import pandas as pd
from sklearn.metrics import precision_recall_curve, average_precision_score
from matplotlib import pyplot


def precision_recall(y_true, y_pred, plot=True, marker='', round_=2):
	if type(y_pred) == pd.Series:
		y_pred_ = []
		y_pred_.append(y_pred)
	else:
		y_pred_ = y_pred
	
	for sc in y_pred_:
		precision, recall, _ = precision_recall_curve(y_true, sc)
		pyplot.plot(recall, precision, marker=marker, label=sc.name)
		
		average = average_precision_score(y_true, sc)
		average = round(average, round_)
		print(f"{sc.name} average precision: ", average)
		
	pyplot.xlabel('Recall')
	pyplot.ylabel('Precision')
	pyplot.legend()
	
	if plot:
		pyplot.show()


from sklearn.metrics import mean_absolute_error, mean_absolute_percentage_error
import pandas as pd


def mean_error(y_true, y_pred):
	'''print beauty table
	'''
	mae = mean_absolute_error(y_true, y_pred)
	mape = mean_absolute_percentage_error(y_true, y_pred)
	
	table = pd.DataFrame()
	table.index = ['MAE', 'MAPE']
	table[' '] = [mae, mape]

	print(table)
	print()
	return mae, mape


import pandas as pd
import pandasflow as pdf

from sklearn.metrics import classification_report


def calc_class_from_score(score:pd.Series, thr=0.5, y_true:pd.Series | None = None) -> pd.Series:
	df = pd.DataFrame({'score': score, 'y_true': y_true})
	
	_thr = thr
	
	if thr in [True, 0, 'best']:
		if y_true is not None:
			_thr = pdf.model.best_thr_f1(df['y_true'], df['score'])
		else:
			print('incorrect thr, y_true is lost')
	
	df['y_pred'] = (df['score'] > _thr) * 1
	
	if y_true is not None:
		print(classification_report(df['y_true'], df['y_pred']))
	
	return df['y_pred']


import pandasflow as pdf
import pandas as pd
import numpy as np



t = pd.DataFrame()
t['y_true'] = [0]*990 + [1]*10
t['y_pred'] = [0]*990 + [1]*10

pdf.metrics.metrics_class(t['y_true'], t['y_pred'])










def get_psqlt():
	print('''
!sudo apt-get -y -qq update
!sudo apt-get -y -qq install postgresql
!sudo service postresqg start

!sudo -u postgres psql -U postgres -c "ALTER USER posgres PASSWORD 'postgres';"

from sqlalchemy import create_engine
con = create_engine('postgresql+psycopg2://postgres:postgres@localhost:5432/postgres')

def select(sql):
  return pd.read_sql(sql, con)

'''[1:])


if __name__ == "__main__":
	get_psqlt()









import pandasflow as pdf
import pandas as pd



df = pd.read_csv('test_data.csv')

train, valid, test = pdf.split.train_valid_test(df, target='Exited', stratify='Exited')
train['age_group'] = pd.cut(train['Age'], [0, 31, 35, 40, 45, float('inf')])
model = train.groupby(['NumOfProducts', 'age_group'])['Exited'].mean().reset_index()
model = model.rename({'Exited': 'score_prod_age'}, axis=1)
train = train.merge(model, how='left', on=['NumOfProducts', 'age_group'])
test['age_group'] = pd.cut(test['Age'], [0, 31, 35, 40, 45, float('inf')])
test = test.merge(model, how='left', on=['NumOfProducts', 'age_group'])
test['y_pred'] = (test['score_prod_age'] > 0.5) * 1


pdf.model.best_thr_f1(test['Exited'], test['score_prod_age'])




import pandasflow as pdf
import pandas as pd
from sklearn.metrics import roc_curve, roc_auc_score
from matplotlib import pyplot


def roc_auc(y_true, y_pred, plot=True, marker='', round_=2):
	if type(y_pred) == pd.Series:
		y_pred_ = []
		y_pred_.append(y_pred)
	else:
		y_pred_ = y_pred
	
	for sc in y_pred_:
		fpr, tpr, _ = roc_curve(y_true, sc)
		pyplot.plot(fpr, tpr, marker=marker, label=sc.name)
		
		auc = roc_auc_score(y_true, sc)
		auc = round(auc, round_)
		print(f"{sc.name}: ", auc)
	
	pyplot.xlabel('False Positive Rate')
	pyplot.ylabel('True Positive Rate')
	pyplot.legend(loc = 'lower right')
	
	if plot:
		pyplot.show()







__version__ = '0.9.3'

requirements = ['pandas', 'scikit-learn', 'catboost', 'phik', 'shap', 'tqdm']




from pandasflow.get_import import get_import

from pandasflow.get_sqlt import get_sqlt

from pandasflow.get_psql import get_psqlt

from pandasflow.reset_mi import reset_mi



# packages
from pandasflow import split

from pandasflow import metrics

from pandasflow import model





from sklearn.metrics import confusion_matrix
import pandas as pd




def conf_mat(y_true, y_pred, labels=None, sample_weight=None, normalize=None):
	"""Compute confusion matrix to evaluate the accuracy of a classification.
	
		Parameters
		----------
		y_true : array-like of shape (n_samples,)
			Ground truth (correct) target values.

		y_pred : array-like of shape (n_samples,)
			Estimated targets as returned by a classifier.

		labels : array-like of shape (n_classes), default=None
			List of labels to index the matrix. This may be used to reorder
			or select a subset of labels.
			If ``None`` is given, those that appear at least once
			in ``y_true`` or ``y_pred`` are used in sorted order.
		
		Returns
		-------
		C :
			TN, FP, FN, TP
	"""
	tn, fp, fn, tp = confusion_matrix(y_true, y_pred, labels=labels, sample_weight=sample_weight, normalize=normalize).ravel()
	
	print(f'TN: {tn}\nFP: {fp}\nFN: {fn}\nTP: {tp}')
	print()
	return tn, fp, fn, tp








import pandasflow as pdf
from test import df
import pandas as pd




train, valid, test = pdf.split.train_valid_test(df)
print('\n')

train, valid, test = pdf.split.train_valid_test(df, target='Exited')
print('\n')

train, valid, test = pdf.split.train_valid_test(df, target='Exited', stratify='Exited')
print('\n')

train, valid, test = pdf.split.train_valid_test(df, target=['Exited', 'Age'])
print('\n')

train, valid, test = pdf.split.train_valid_test(df, target=['Exited', 'Age'], stratify='Exited')
print('\n')

train, valid, test = pdf.split.train_valid_test(df, target=['Exited', 'Age'], stratify='Exited', round_=3)
print('\n')

train, valid, test = pdf.split.train_valid_test(df, target=['Exited', 'Age'], stratify='Exited', round_=1)
print('\n')





def get_import():
	print('''
import sqlite3
import pandas as pd
import numpy as np
import seaborn as sns
import matplotlib.pyplot as plt
import sklearn
import phik
import shap

from catboost import CatBoostClassifier, CatBoostRegressor
from catboost import Pool

plt.style.use("dark_background")
shap.initjs()
'''[1:])


if __name__ == "__main__":
	get_import()


import pandasflow as pdf

import pandas as pd


def inherit_train_test(
	df: pd.DataFrame,
	train_size=0.6,
	round_=None):
	
	train_size_pice = int(len(df) * train_size)
	
	train = df.iloc[:train_size_pice]
	test = df.iloc[train_size_pice:]
	
	train_pie = len(train) / len(df)
	test_pie = len(test) / len(df)
	
	amount_len = len(train) + len(test)
	amount_prop = amount_len / len(df)
	
	if round_ not in [None, 'n', 'N'] and round_ > 0:
		train_pie = round(train_pie, round_)
		test_pie = round(test_pie, round_)
	
	table = pd.DataFrame({
		'count': [len(train),
				  len(test),
				  '',
				  amount_len,
				  len(df)],
		
		'pct': [train_pie,
				test_pie,
				'',
				amount_prop,
				''],
	})
	
	table.index = ['train',
				   'test',
				   '',
				   'Amount',
				   'InitData']
	print(table)
	
	return train, test




